##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-25
# About: Query and Wrangle Data on Age and Sex from 2011 census, all GB


# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- TRUE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?mode=construct&version=0&dataset=1414
# based on the address shown in the browser we know that "dataset=1414"
# LC1117EW
  
# scan data set
nomisr::nomis_get_metadata(id = "NM_1414_1") # for a general overview of categories 

# check the geography level category
nomisr::nomis_get_metadata(id = "NM_1414_1", concept = "GEOGRAPHY", type = "type")
# we will need TYPE298

# check the age category
nomisr::nomis_get_metadata(id = "NM_1414_1", concept = "C_AGE")
# we will need  17:39

# check the sex category
nomisr::nomis_get_metadata(id = "NM_1414_1", concept = "C_SEX")
# we will need  1:2

# check the measure category
nomisr::nomis_get_metadata(id = "NM_1414_1", concept = "MEASURES")
# we will need 20100

# check the freq category
nomisr::nomis_get_metadata(id = "NM_1414_1", concept = "FREQ")
# we will need A

# check the time category
nomisr::nomis_get_metadata(id = "NM_1414_1", concept = "TIME")
# we will need 2011 

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_1414_1",
                         geography = c("TYPE298"),
                         c_age = c(17:39),
                         c_sex = c(1:2),
                         measures = c(20100),
                         freq = c("A"),
                         time = c(2011), 
                         tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- #

### Bring into correct format: EW age and sex ###

data_EW <- copy(data_EW[, .(areacode = geography_code, 
                            sex      = c_sex_name,
                            age_code = c_age_name, 
                            observations = obs_value)])
data_EW[age_code %in% c("Age 16", "Age 17", "Age 18", "Age 19", "Age 20",
                        "Age 21", "Age 22", "Age 23", "Age 24"), age_cat :=  "16_24"]
data_EW[age_code %in% c("Age 25 to 29", "Age 30 to 34"), age_cat :=  "25_34"]
data_EW[age_code %in% c("Age 35 to 39", "Age 40 to 44", "Age 45 to 49"), age_cat :=  "35_49"]
data_EW[age_code %in% c("Age 50 to 54", "Age 55 to 59", "Age 60 to 64"), age_cat :=  "50_64"]
data_EW[age_code %in% c("Age 65 to 69", "Age 70 to 74"), age_cat :=  "65_74"]
data_EW[age_code %in% c("Age 75 to 79", "Age 80 to 84", "Age 85 to 89", "Age 90 and over"),
        age_cat :=  "75p"]

data_EW[, observations_age_cat := sum(observations),
        by = c("areacode", "sex", "age_cat")]

data_EW <- unique(data_EW[, .(areacode, sex, age_cat, observations_age_cat)])
data_EW[, sex := tolower(substr(sex, 1, 1))]
data_EW[, age_sex := paste0(sex,"_",age_cat)]
data_EW <- data_EW[, .(areacode, observations = observations_age_cat, age_sex)]
data_EW <- data.table::dcast(data_EW, areacode ~ age_sex, 
                             value.var = "observations")
data_EW[, total := rowSums(.SD), .SDcols = 2:(dim(data_EW)[2])]

# ---------------------------------------------------------------------------- #

##### Scotland #####

# identify file paths from here: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                     "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
              destfile = "RData/sco_2011.zip", mode = "wb", timeout = 6000000)
# unzip file 
unzip(zipfile = "RData/sco_2011.zip",
      exdir = "RData/")
}
# load file
data_SCO <- data.table::fread("RData/LC1117SC.csv", header = TRUE)
data.table::setnames(data_SCO, old = names(data_SCO),
                     new = c("areacode", "age_code", "all", "males", "females"))
data_SCO <- data_SCO[!(areacode %in% c("S92000003","")) & 
                     !(age_code %in% c("All people", "0 to 4", "5 to 9",
                                  "10 to 11", "12 to 14", "15")),
                .(areacode, age_code, males, females)]


data_SCO[age_code %in% c("16 to 17", "18 to 19", "20 to 24"), age_cat := "16_24"]
data_SCO[age_code %in% c("25 to 29", "30 to 34"), age_cat := "25_34"]
data_SCO[age_code %in% c("35 to 39", "40 to 44", "45 to 49"), age_cat := "35_49"]
data_SCO[age_code %in% c("50 to 54", "55 to 59", "60 to 64"), age_cat := "50_64"]
data_SCO[age_code %in% c("65 to 69", "70 to 74"), age_cat := "65_74"]
data_SCO[age_code %in% c("75 to 79", "80 to 84", "85 to 89", "90 to 94", "95 and over"),
        age_cat :=  "75p"]

data_SCO <- data_SCO[, .(males = sum(males), females = sum(females)),
                     by = c("areacode", "age_cat")]

data_SCO <- melt(data_SCO, id.vars = c("areacode", "age_cat"),
                 variable.name = "sex", value.name = "observations")

data_SCO[, age_sex := paste0(substr(sex, 1, 1),"_",age_cat)]
data_SCO <- data_SCO[, .(areacode, age_sex, observations)]
data_SCO <- data.table::dcast(data_SCO, areacode ~ age_sex, 
                             value.var = "observations")
data_SCO[, total := rowSums(.SD), .SDcols = 2:(dim(data_SCO)[2])]

# ---------------------------------------------------------------------------- #

##### Combine E&W and Scotland #####
data_GB <- rbind(data_EW, data_SCO)
data.table::fwrite(data_GB, "ROutput/2011_census_age_sex.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #