##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-05
# About: Query and Wrangle Data on Ethnicity and Economic Activity from 2021 census, all GB

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- FALSE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?reset=yes&mode=construct&dataset=818&version=0&anal=1&initsel=
# based on the address shown in the browser we know that "dataset=818""

# scan data set
nomisr::nomis_get_metadata(id = "NM_818_1") # for a general overview of categories 

# check the geography level category
nomisr::nomis_get_metadata(id = "NM_818_1", concept = "GEOGRAPHY", type = "type")
# we will need TYPE298

# check the time category
nomisr::nomis_get_metadata(id = "NM_818_1", concept = "TIME")
# we will need 2011 

# check the age category
nomisr::nomis_get_metadata(id = "NM_818_1", concept = "C_AGE")
# we will need  1 and 2

# check the measure category
nomisr::nomis_get_metadata(id = "NM_818_1", concept = "MEASURES")
# we will need 20100

# check economic activity category
nomisr::nomis_get_metadata(id = "NM_818_1", concept = "C_ECOPUK11")
# we will need 0, 3, 6, 9, 10, 14, 15, 16, 17, 18

# ethnicity 
nomisr::nomis_get_metadata(id = "NM_818_1", concept = "C_ETHPUK11")
# we will need 0,1,5,6,7,8

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_818_1", geography = "TYPE298",
                         time = c(2011), 
                         c_ecopuk11 = c(0, 3, 6, 9, 10, 14, 15, 16, 17, 18),
                         c_ethpuk11 = c(0, 1, 5, 6, 7, 8),
                         c_age = c(0), measures = 20100, tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- #

### Bring into correct format: EW ethnicity ###

data_ethnicity_EW <- copy(data_EW[c_ecopuk_11_name == "All categories: Economic activity",
                                  .(Areacode = geography_code, 
                                   ethnicity = c_ethpuk_11_name,
                                   observations = obs_value)])
data_ethnicity_EW <- data.table::dcast(data_ethnicity_EW, Areacode ~ ethnicity,
                           value.var = "observations")
setkey(data_ethnicity_EW, Areacode)
data.table::setcolorder(data_ethnicity_EW,
                        c(names(data_ethnicity_EW[,1]),
                          names(data_ethnicity_EW[,3:dim(data_ethnicity_EW)[2]]),
                          names(data_ethnicity_EW[,2])))
data_ethnicity_EW[, N_total_test := rowSums(.SD), .SDcols = 2:(dim(data_ethnicity_EW)[2]-1)]

# ---------------------------------------------------------------------------- #

### Bring into correct format: EW economic activity ###

data_economic_EW <- unique(copy(data_EW[c_ecopuk_11_name != "Economically active: Total" 
                                        & c_ethpuk_11_name ==	"All categories: Ethnic group",
                              .(Areacode = geography_code, 
                                economic = c_ecopuk_11_name,
                                observations = obs_value)]))
data_economic_EW <- data.table::dcast(data_economic_EW, Areacode ~ economic,
                                       value.var = "observations")
setkey(data_economic_EW, Areacode)
data.table::setcolorder(data_economic_EW,
                        c(names(data_economic_EW[,1]),
                          names(data_economic_EW[,3:dim(data_economic_EW)[2]]),
                          names(data_economic_EW[,2])))
data_economic_EW[, N_total_test := rowSums(.SD), .SDcols = 2:(dim(data_economic_EW)[2]-1)]


# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Scotland #####

# identify file paths from here: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                     "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
              destfile = "RData/sco_2011.zip", mode = "wb", timeout = 6000000)
# unzip file 
unzip(zipfile = "RData/sco_2011.zip",
      exdir = "RData/")
}
# load file
data_SCO <- data.table::fread("RData/LC6201SC.csv", header = TRUE)
data_SCO <- data_SCO[16:dim(data_SCO)[1],]
data_SCO <- janitor::clean_names(data_SCO)

# ---------------------------------------------------------------------------- # 

### Bring into correct format: SCO ethnicity ###

data_ethnicity_SCO <- copy(data_SCO[v2 == "All people aged 16 and over",])
data_ethnicity_SCO[, v2 := NULL]
data.table::setnames(data_ethnicity_SCO, old = "v1", new = "Areacode")
data.table::setcolorder(data_ethnicity_SCO,
                      c(names(data_ethnicity_SCO[,1]),
                        names(data_ethnicity_SCO[,3:dim(data_ethnicity_SCO)[2]]),
                        names(data_ethnicity_SCO[,2])))
data_ethnicity_SCO[, N_total_test := rowSums(.SD),
                   .SDcols = 2:(dim(data_ethnicity_SCO)[2]-1)]

# ---------------------------------------------------------------------------- # 

### Bring into correct format: SCO employment ###

data_economic_SCO <- copy(data_SCO[ 
      v2 %in% c("All people aged 16 and over",
                "Economically active: Employee: Total",
                "Economically active: Self-employed: Total",
                "Economically active: Unemployed (1)",
                "Economically inactive: Retired",
                "Economically inactive: Student",
                "Economically inactive: Looking after home or family",
                "Economically inactive: Long-term sick or disabled",
                "Economically inactive: Other"),
      .(v1, v2, all_people_aged_16_and_over)])
data.table::setnames(data_economic_SCO, old = names(data_economic_SCO), 
                     new = c("Areacode","economic", "observations"))                                   
data_economic_SCO <- data.table::dcast(data_economic_SCO, Areacode ~ economic,
                                       value.var = "observations")
data.table::setcolorder(data_economic_SCO,
                        c(names(data_economic_SCO[,1]),
                          names(data_economic_SCO[,3:dim(data_economic_SCO)[2]]),
                          names(data_economic_SCO[,2])))
data_economic_SCO[, N_total_test := rowSums(.SD),
                  .SDcols = 2:(dim(data_economic_SCO)[2]-1)]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Ethnicity: Combine into One Constraint File #####

# Recode England and Wales #
data.table::setnames(data_ethnicity_EW, 
                     old = names(data_ethnicity_EW),
                     new = c("areacode", "asian", "black", "mixed", "other",
                             "white", "total", "total2"))
data_ethnicity_EW[, total2 := NULL]
data.table::setcolorder(data_ethnicity_EW,
            c("areacode", "white", "mixed", "asian", "black", "other", "total"))


# Recode Scotland #
data_ethnicity_SCO[, black := african + caribbean_or_black]
data.table::setnames(data_ethnicity_SCO, 
                     old = names(data_ethnicity_SCO),
                     new = c("areacode", "white", "mixed", "asian", "african",
                             "caribbean", "other", "total", "total2", "black"))
data_ethnicity_SCO[, c("african", "caribbean", "total2") := NULL]
data.table::setcolorder(data_ethnicity_SCO,
                        c("areacode", "white", "mixed", "asian", "black", "other", "total"))

# combine Scotland and EW + Save # 
data_ethnicity_GB <- rbind(data_ethnicity_EW, data_ethnicity_SCO)
data.table::fwrite(data_ethnicity_GB, "ROutput/2011_census_ethnicity.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Economic Activity: Combine into One Constraint File #####

# Recode England and Wales #
data_economic_EW <- janitor::clean_names(data_economic_EW)

# collapse in_paid_employment
data_economic_EW[, in_paid_employment := 
                   economically_active_in_employment_employee_total + 
                   economically_active_in_employment_full_time_students]
data_economic_EW[, c("economically_active_in_employment_employee_total",
                     "economically_active_in_employment_full_time_students") := NULL]

data.table::setnames(data_economic_EW, 
                     old = names(data_economic_EW),
                     new = c("areacode", "self_employed", "unemployed",
                             "long_term_sick_disabled", "looking_after_home",
                             "other", "retired", "student", "total", "total2",
                             "in_paid_employment"))
data_economic_EW[, total2 := NULL]
data.table::setcolorder(data_economic_EW,
                        c("areacode", "self_employed", "in_paid_employment",
                          "unemployed", "retired", "other", "looking_after_home",
                          "student", "long_term_sick_disabled", "total"))

# -------------- #

# Recode Scotland #
data_economic_SCO <- janitor::clean_names(data_economic_SCO)
data.table::setnames(data_economic_SCO, 
                     old = names(data_economic_SCO),
                     new = c("areacode", "in_paid_employment", "self_employed",
                             "unemployed", "long_term_sick_disabled", 
                             "looking_after_home", "other", "retired",
                             "student", "total", "total2"))
data_economic_SCO[, total2 := NULL]
data.table::setcolorder(data_economic_SCO,
                        c("areacode", "self_employed", "in_paid_employment",
                          "unemployed", "retired", "other", "looking_after_home",
                          "student", "long_term_sick_disabled", "total"))
                             
# combine Scotland and EW + Save # 
data_economic_GB <- rbind(data_economic_EW, data_economic_SCO)
data.table::fwrite(data_economic_GB, "ROutput/2011_census_economic.csv")
# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #
                       
