##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-25
# About: Query and Wrangle Data on General Health from 2011 census, all GB
# Table EW: QS302EW
# Table SCO: QS302SCO

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- FALSE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?reset=yes&mode=construct&dataset=531&version=0&anal=1&initsel=
# based on the address shown in the browser we know that "dataset=531""

# scan data set
nomisr::nomis_get_metadata(id = "NM_531_1") # for a general overview of categories 

# check the geography level category
nomisr::nomis_get_metadata(id = "NM_531_1", concept = "GEOGRAPHY", type = "type")
# we will need TYPE298

# check the urban rural category
nomisr::nomis_get_metadata(id = "NM_531_1", concept = "RURAL_URBAN")
# we will need  0

# check the age category
nomisr::nomis_get_metadata(id = "NM_531_1", concept = "C_HEALTH")
# we will need  1:5

# check the measure category
nomisr::nomis_get_metadata(id = "NM_531_1", concept = "MEASURES")
# we will need 20100

# check the time category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "FREQ")
# we will need A

# check the time category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "TIME")
# we will need 2011 

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_531_1", geography = "TYPE298",
                         rural_urban = c(0),
                         c_health = c(1, 2, 3, 4, 5),
                         measures = c(20100),
                         freq = c("A"),
                         time = c(2011), 
                         tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- #

### Bring into correct format: EW marital status ###
data_EW[
  c_health == 1, general_health := "very_good_health"][ 
  c_health == 2, general_health := "good_health"][ 
  c_health == 3, general_health := "fair_health"][
  c_health == 4 | c_health == 5, general_health := "poor_health"]
  
data_EW <- copy(data_EW[, .(observations = sum(obs_value)),
                            by = c("geography_code", "general_health")])
data.table::setnames(data_EW,
                     old = "geography_code", new = "areacode")
data_EW <- data.table::dcast(data_EW, areacode ~ general_health,
                           value.var = "observations")
data_EW[, total := rowSums(.SD), .SDcols = 2:(dim(data_EW)[2])]

# ---------------------------------------------------------------------------- #

##### Scotland #####

# identify file paths from here: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                     "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
              destfile = "RData/sco_2011.zip", mode = "wb", timeout = 6000000)
# unzip file 
unzip(zipfile = "RData/sco_2011.zip",
      exdir = "RData/")
}
# load file
data_SCO <- data.table::fread("RData/QS302SC.csv", header = TRUE)
data.table::setnames(data_SCO, old = names(data_SCO),
                     new = c("areacode", "total", "very_good_health", "good_health", "fair_health",
                             "poor_health_1", "poor_health_2"))
data_SCO[, poor_health := poor_health_1 + poor_health_2]
data_SCO <- data_SCO[!(areacode %in% c("S92000003","")),
                                  .(areacode, very_good_health, good_health, fair_health,
                                    poor_health)]
data_SCO[, total := rowSums(.SD), .SDcols = 2:(dim(data_SCO)[2])]

# ---------------------------------------------------------------------------- #

##### Combine E&W and Scotland #####
data_GB <- rbind(data_EW, data_SCO)

# ---------------------------------------------------------------------------- #

##### Apply 0-15 Correction #####

# as this is for all ages, we apply a correction to substract individuals aged 0-15
# for this purpose, we use the master constraint Age-Sex and weight observed numbers
data_master <- data.table::fread("RData/2011_census_age_sex.csv", header = TRUE)
data_master <- data_master[, .(areacode, total_16plus = total)]
data_master <- merge(data_master, data_GB, by = "areacode")

# as this is for all ages, we apply a correction to substract individuals aged 0-15
data_master[, p := round(total_16plus / total, 4)]
data_master[, very_good_health := trunc(very_good_health * p)]
data_master[, good_health := trunc(good_health * p)]
data_master[, fair_health := trunc(fair_health * p)]
data_master[, poor_health  := trunc(poor_health * p)]
data_master[, c("total", "total_16plus", "p") := NULL]
data_master[, total := rowSums(.SD), .SDcols = 2:(dim(data_master)[2])]

data.table::setcolorder(data_master, c("areacode", "very_good_health", "good_health",
                                       "fair_health", "poor_health", "total"))

# ---------------------------------------------------------------------------- #

##### Save all Data for E&W and Scotland #####

data.table::fwrite(data_master, "ROutput/2011_census_general_health.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #