##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-25
# About: Query and Wrangle Data on HH composition from NOMIS / NRS, all GB
# England and Wales: LC1104EW
# Scotland: LC1104SCO
# NOTE: The data covers all age groups and we identify and exclude 0-16y
# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- FALSE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?reset=yes&mode=construct&dataset=849&version=0&anal=1&initsel=
# based on the address shown in the browser we know that "dataset=818""

# scan data set
nomisr::nomis_get_metadata(id = "NM_849_1") # for a general overview of categories 

# check the geography level category
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "GEOGRAPHY", type = "type")
# we will need TYPE298

# check the sex category
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "C_SEX")
# we will need 0

# check the age category
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "C_AGE")
# we will need  2,3,4,5

# check the HH composition category .... this needs sorted .... 
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "C_HHCHUK11")
# •	one.person.no.child         -->  1
# •	one.person.with.children    --> 14
# •	married.no.child            -->  7
# •	married.with.children       --> 8+9
# •	cohibiting.no.child         --> 11    
# •	cohibiting.with.children    --> 12+13
# •	other.family.no.child       --> 5 
# •	other.family.with.children  --> ??? // THERE IS NO "other.family.with.children"
# •	other.not.family.no.child   --> 19
# •	other.not.family.with.children --> 18
# not used: 0, 2, 3, 4, 6, 10, 15, 16, 17
# used: 1, 5, 7, 8, 9, 11, 12, 13, 14, 18, 19

# check the measure category
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "MEASURES")
# we will need 20100

# check the frequency category
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "FREQ")
# we will need A 

# check the time category
nomisr::nomis_get_metadata(id = "NM_849_1", concept = "TIME")
# we will need 2011 

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_849_1",
                         geography = "TYPE298",
                         c_sex = c(0), 
                         c_age = c(2,3,4,5),
                         c_hhchuk11 = c(1, 5, 7, 8, 9, 11, 12, 13, 14, 18, 19),
                         measures = 20100,
                         freq = "A",
                         time = c(2011),
                         tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- #

### Bring into correct format: EW composition ###

data_EW <- copy(data_EW[, .(Areacode = geography_code, 
                                   composition = c_hhchuk_11,
                                   age = c_age_name,
                                   observations = obs_value)])
data_EW <- data_EW[, .(observations = sum(observations)),
                               by = c("Areacode","composition")]
data_EW[
  composition == 1, hh_comp := "one_person_no_child"][
  composition == 14, hh_comp := "one_person_with_children"][ 
  composition == 7, hh_comp := "married_no_child"][ 
  composition == 8 | composition == 9, hh_comp := "married_with_children"][
  composition == 11, hh_comp := "cohibiting_no_child"][
  composition == 12 | composition == 13, hh_comp := "cohibiting_with_children"][
  composition == 5, hh_comp := "other_family_no_child"][
  composition == 19, hh_comp := "other_not_family_no_child"][
  composition == 18, hh_comp := "other_not_family_with_children"][,
    composition := NULL]
data.table::setcolorder(data_EW, c("Areacode","hh_comp", "observations"))
data_EW <- data_EW[, .(observations = sum(observations)),
                                           by = c("Areacode","hh_comp")]
data_EW <- data.table::dcast(data_EW, Areacode ~ hh_comp,
                           value.var = "observations")
data_EW[, total := rowSums(.SD), .SDcols = 2:(dim(data_EW)[2])]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Scotland ##### 

# identify file paths from here: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                     "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
              destfile = "RData/sco_2011.zip", mode = "wb", timeout = 6000000)
# unzip file 
unzip(zipfile = "RData/sco_2011.zip",
      exdir = "RData/")
}
# load file
data_SCO <- data.table::fread("RData/LC1109SC.csv", header = TRUE)
data_SCO <- janitor::clean_names(data_SCO)

# ---------------------------------------------------------------------------- # 

data_SCO <- data_SCO[!(v1 %in% c("S92000003","")) & 
                                             !(v2 %in% c("All people in households",
                                                         "0 to 15")), ]
names_SCO <- cbind(names(data_SCO))
# done: 4, 5, 6, 7, 8,  9, 10,
# done: 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25 
# del:  4,5,6,7,8,9,10,11,13,14,15,16,17,18,19,20
# one_person_no_child #
data_SCO[, one_person_no_child := one_person_household_total][,
  c("one_person_household_total",
    "one_person_household_aged_65_and_over",
    "one_person_household_aged_under_65") := NULL]

# one_person_with_children
data_SCO[, one_person_with_children := one_family_household_lone_parent_total][,
   c("one_family_household_total",
     "one_family_household_lone_parent_total",
     "one_family_household_lone_parent_with_dependent_children",
     "one_family_household_lone_parent_all_children_non_dependent") := NULL]

# married_no_child
data_SCO[, married_no_child := one_family_household_married_couple_no_children +
                                           one_family_household_same_sex_civil_partnership_couple][,
  c("one_family_household_married_couple_no_children",
    "one_family_household_same_sex_civil_partnership_couple") := NULL]

# married_with_children
data_SCO[, married_with_children := one_family_household_married_couple_with_dependent_children +
                                                one_family_household_married_couple_all_children_non_dependent][,
        c("one_family_household_married_couple_total",
          "one_family_household_married_couple_with_dependent_children",
          "one_family_household_married_couple_all_children_non_dependent") := NULL]

# cohibiting_no_child
data_SCO[, cohibiting_no_child := one_family_household_cohabiting_couple_no_children][,
  c("one_family_household_cohabiting_couple_no_children",
    "one_family_household_cohabiting_couple_total") := NULL]

# cohibiting_with_children
data_SCO[, cohibiting_with_children := one_family_household_cohabiting_couple_with_dependent_children +
                       one_family_household_cohabiting_couple_all_children_non_dependent][,
       c("one_family_household_cohabiting_couple_with_dependent_children",
         "one_family_household_cohabiting_couple_all_children_non_dependent") := NULL]

# other_family_no_child
data_SCO[, other_family_no_child := one_family_household_all_aged_65_and_over][,
       one_family_household_all_aged_65_and_over := NULL]

# other_not_family_no_child
data_SCO[, other_not_family_no_child := other_household_types_all_full_time_students + 
                                                    other_household_types_all_aged_65_and_over + 
                                                    other_household_types_other][,
  c("other_household_types_total",
    "other_household_types_all_full_time_students",
    "other_household_types_all_aged_65_and_over",
    "other_household_types_other") := NULL]

# other_not_family_with_children
data_SCO[, other_not_family_with_children := other_household_types_with_dependent_children][,
  "other_household_types_with_dependent_children" := NULL]

data_SCO <- data_SCO[, .(
                             one_person_no_child = sum(one_person_no_child),
                             one_person_with_children = sum(one_person_with_children),
                             married_no_child  = sum(married_no_child),
                             married_with_children  = sum(married_with_children),
                             cohibiting_no_child = sum(cohibiting_no_child),
                             cohibiting_with_children = sum(cohibiting_with_children),
                             other_family_no_child = sum(other_family_no_child),
                             other_not_family_no_child = sum(other_not_family_no_child),
                             other_not_family_with_children  = sum(other_not_family_with_children)),
                         by = c("v1")]
data.table::setnames(data_SCO, old = "v1", new = "Areacode")  
data_SCO[, total := rowSums(.SD), .SDcols = 2:(dim(data_SCO)[2])]
data.table::setcolorder(data_SCO, c(names(data_EW)))

# ---------------------------------------------------------------------------- #

##### Combine E&W and Scotland #####
data_GB <- rbind(data_EW, data_SCO)
data.table::setnames(data_GB, old = "Areacode", new = "areacode")  
data.table::setnames(data_GB, old = "cohibiting_no_child",
                     new = "cohabiting_no_child")  
data.table::setnames(data_GB, old = "cohibiting_with_children",
                     new = "cohabiting_with_children")  
data.table::fwrite(data_GB, "ROutput/2011_census_hh_tenure.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

