##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-25
# About: Query and Wrangle Data on HH tenure for individuals from 2011 census, all GB
# NOTE: The queried data for Scotland includes the entire age range so we apply an adjustment

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- FALSE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?reset=yes&mode=construct&dataset=1403&version=0&anal=1&initsel=
# based on the address shown in the browser we know that "dataset=1403""

# scan data set
nomisr::nomis_get_metadata(id = "NM_1403_1") # for a general overview of categories 

# check the geography level category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "GEOGRAPHY", type = "type")
# we will need TYPE298

# check the age category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "C_TENHUK11")
# we will need  2 --> ownedOutright
# we will need  3 --> OwnedMorgage
# we will need  5 --> SocialRented
# we will need  6 --> PrivateRented

# check the age category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "C_AGE")
# we will need  2,3,4

# check the disability category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "C_DISABILITY")
# we will need 0

# check the measure category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "MEASURES")
# we will need 20100

# check the time category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "FREQ")
# we will need A

# check the time category
nomisr::nomis_get_metadata(id = "NM_1403_1", concept = "TIME")
# we will need 2011 

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_1403_1",
                         geography = "TYPE298",
                         c_tenhuk11 = c(2, 3, 5, 6),
                         c_age = c(2, 3, 4), 
                         c_disability = c(0), 
                         measures = c(20100),
                         freq = c("A"),
                         time = c(2011), 
                         tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- # 

### Bring into correct format: EW tenure ###
data_EW[
  c_tenhuk_11 == 2, tenure := "OwnedOutright"][
  c_tenhuk_11 == 3, tenure := "OwnedMortgage"][ 
  c_tenhuk_11 == 5, tenure := "SocialRented"][ 
  c_tenhuk_11 == 6, tenure := "PrivateRented"]

data_EW <- copy(data_EW[, .(observations = sum(obs_value)),
                            by = c("geography_code","tenure")])

data.table::setnames(data_EW,
                     old = "geography_code", new = "Areacode")
data_EW <- data.table::dcast(data_EW, Areacode ~ tenure,
                           value.var = "observations")
data_EW[, total := rowSums(.SD), .SDcols = 2:(dim(data_EW)[2])]

# ---------------------------------------------------------------------------- #

##### Scotland #####

# identify file paths from here: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                     "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
              destfile = "RData/sco_2011.zip", mode = "wb", timeout = 6000000)
# unzip file 
unzip(zipfile = "RData/sco_2011.zip",
      exdir = "RData/")
}

# load file
data_SCO <- data.table::fread("RData/QS403SC.csv", header = TRUE)
data.table::setnames(data_SCO, old = names(data_SCO),
                     new = c("Areacode", "Total", "OwnedTotal", "OwnedOutright",
                     "OwnedMortgage", "OwnedShared", "SocialRentedTotal",
                     "SocialRentedLA", "SocialRentedOther","PrivateRentedTotal",
                     "PrivateRentedLandlord", "PrivateRentedEmployer",
                     "PrivateRentedFamily", "PrivateRentedOther",
                     "RentFree"))
data_SCO <- data_SCO[!(Areacode %in% c("S92000003","")), ]
data_SCO[, c("Total","OwnedTotal","SocialRentedLA", "SocialRentedOther",
                    "PrivateRentedLandlord", "PrivateRentedEmployer",
                    "PrivateRentedFamily", "PrivateRentedOther") := NULL]
data_SCO[, PrivateRentedTotal := PrivateRentedTotal + RentFree + OwnedShared]
data_SCO[, c("RentFree","OwnedShared") := NULL]                                                                                     
data_SCO[, total := rowSums(.SD), .SDcols = 2:(dim(data_SCO)[2])]
data.table::setnames(data_SCO, old = "SocialRentedTotal", new = "SocialRented")
data.table::setnames(data_SCO, old = "PrivateRentedTotal", new = "PrivateRented")
data.table::setcolorder(data_SCO, c("Areacode", "OwnedMortgage",
                                    "OwnedOutright", "PrivateRented", "SocialRented", "total"))
# ---------------------------------------------------------------------------- #

##### Combine E&W and Scotland #####
data_GB <- rbind(data_EW, data_SCO)
data.table::setnames(data_GB, old = names(data_GB),
                     new = c("areacode", "owned_mortage", "owned_outright",
                             "private_rented", "social_rented", "total"))

# ---------------------------------------------------------------------------- #

##### Apply 0-16y adjustment as data currently covers all ages: only for Scotland #####

data_master <- data.table::fread("RData/2011_census_age_sex.csv",  header = TRUE,
                                 select = c("areacode", "total"))
data.table::setnames(data_master, old = "total", new = "total_16plus")
data_master <- merge(data_GB, data_master, by = "areacode")

# as this is for all ages, we apply a correction to substract individuals aged 0-15
data_master[, p := round(total_16plus / total, 4)]
data_master[substr(areacode,1,1) %in% c("E","W"), p := 1]
  
data_master[, owned_outright := trunc(owned_outright * p)]
data_master[, owned_mortage := trunc(owned_mortage * p)]
data_master[, social_rented := trunc(social_rented * p)]
data_master[, private_rented  := trunc(private_rented * p)]

data_master[, c("total", "total_16plus", "p") := NULL]
data_master[, total := rowSums(.SD), .SDcols = 2:(dim(data_master)[2])]

# ---------------------------------------------------------------------------- #

##### Save all Data for E&W and Scotland #####

data.table::fwrite(data_master, "ROutput/2011_census_hh_tenure.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #




