##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-05
# About: Query and Wrangle Data on Qualificatiion from 2021 census, all GB


# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- FALSE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?reset=yes&mode=construct&dataset=554&version=0&anal=1&initsel=
# based on the address shown in the browser we know that "dataset=603""

# scan data set
nomisr::nomis_get_metadata(id = "NM_554_1") # for a general overview of categories 

# check the geography level category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "GEOGRAPHY", type = "type")
# we will need TYPE298

# check the age category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "RURAL_URBAN")
# we will need  0

# check the age category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "CELL")
# we will need  1:7

# check the measure category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "MEASURES")
# we will need 20100

# check the time category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "FREQ")
# we will need A

# check the time category
nomisr::nomis_get_metadata(id = "NM_554_1", concept = "TIME")
# we will need 2011 

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_554_1", geography = "TYPE298",
                         rural_urban = c(0),
                         cell = c(1, 2, 3, 4, 5, 6, 7),
                         measures = c(20100),
                         freq = c("A"),
                         time = c(2011), 
                         tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- #

### Bring into correct format: EW marital status ###
data_EW[
  cell == 1, education := "none"][ 
  cell == 2 | cell == 3, education := "level_1_to_2"][ 
  cell == 4 | cell == 5 | cell == 7 , education := "level_3"][ 
  cell == 6, education := "level_4_and_above"]
  
data_education_EW <- copy(data_EW[, .(observations = sum(obs_value)),
                            by = c("geography_code", "education")])
data.table::setnames(data_education_EW,
                     old = "geography_code", new = "areacode")
data_education_EW <- data.table::dcast(data_education_EW, areacode ~ education,
                           value.var = "observations")
data_education_EW[, total := rowSums(.SD), .SDcols = 2:(dim(data_education_EW)[2])]

# ---------------------------------------------------------------------------- #

##### Scotland #####

# identify file paths from here: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                     "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
              destfile = "RData/sco_2011.zip", mode = "wb", timeout = 6000000)
# unzip file 
unzip(zipfile = "RData/sco_2011.zip",
      exdir = "RData/")
}
# load file
data_education_SCO <- data.table::fread("RData/QS501SC.csv", header = TRUE)
data.table::setnames(data_education_SCO, old = names(data_education_SCO),
                     new = c("areacode", "total", "none", "level_1", "level_2",
                             "level_3", "level_4_and_above"))
data_education_SCO[, level_1_to_2 := level_1 + level_2]
data_education_SCO <- data_education_SCO[!(areacode %in% c("S92000003","")),
                                  .(areacode, none, level_1_to_2, level_3,
                                    level_4_and_above)]
data_education_SCO[, total := rowSums(.SD), .SDcols = 2:(dim(data_education_SCO)[2])]

# ---------------------------------------------------------------------------- #

##### Combine E&W and Scotland #####
data_education_GB <- rbind(data_education_EW, data_education_SCO)
data.table::fwrite(data_education_GB, "ROutput/2011_census_qualification.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #