##### Meta #####

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2023-07-05
# About: Query and Wrangle Data on Age and Sex 

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Preparation #####

# clean workspace
rm(list = ls())  # remove all objects from work space 
gc(full = TRUE)  # deep clean garbage
options(timeout = 6000000)  # make timeout larger

# start benchmark time 
benchmark_time <- list() 
benchmark_time$start <- Sys.time()

# List of Required Libraries 
RequiredPackages <- c("data.table", "janitor", "zoo", "nomisr", "readxl")                        

# ensure all packages are installed and loaded 
.EnsurePackages <- function(packages_vector) {
  new_package <- packages_vector[!(packages_vector %in% 
                                     installed.packages()[, "Package"])]
  if (length(new_package) != 0) {
    install.packages(new_package) }
  sapply(packages_vector, suppressPackageStartupMessages(require),
         character.only = TRUE)
}
.EnsurePackages(RequiredPackages)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Definitions #####

# initialize ...
definitions <- list()

# set seed 
definitions$run_queries <- FALSE

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### England and Wales #####

### Explore and Build NOMIS Query ###

if (definitions$run_queries == TRUE) {

# the online query by hand would be from the following address: 
# https://www.nomisweb.co.uk/query/construct/summary.asp?mode=construct&version=0&dataset=2010
# based on the address shown in the browser we know that "dataset=818""

# scan data set
nomisr::nomis_get_metadata(id = "NM_2010") # for a general overview of categories 

# check the geography level category
test <- nomisr::nomis_get_metadata(id = "NM_2010", concept = "GEOGRAPHY")
# we will need TYPE298

# check the HH composition category
nomisr::nomis_get_metadata(id = "NM_2010", concept = "GENDER")
# we will need 1 and 2

# check the time category
nomisr::nomis_get_metadata(id = "NM_2010", concept = "TIME")
# we will need 2020 

# check the age category
test <- nomisr::nomis_get_metadata(id = "NM_2010", concept = "C_AGE")
# we will need  117, 118, 119, 120, 6 --> 16_24
# we will need  7, 8                  --> 25_34
# we will need  9, 10, 11             --> 35-49
# we will need 12, 13, 14             --> 50-64
# we will need 15, 16                 --> 65-74
# we will need 17, 18, 210            --> 75+

# check the measure category
nomisr::nomis_get_metadata(id = "NM_2010", concept = "MEASURES")
# we will need 20100

# -------------- #

## Build Query ###

data_EW <- data.table::data.table(
  nomisr::nomis_get_data(id = "NM_2010", geography = "TYPE298",
                         time = c(2020), gender = c(1, 2), 
                         c_age = c(117, 118, 119, 120, 6, # --> 16_24
                                   7, 8,                  # --> 25_34
                                   9, 10, 11,             # --> 35_49
                                   12, 13, 14,            # --> 50_64
                                   15, 16,                # --> 65_74
                                   17, 18, 210),          # --> 75p
                         measures = 20100, tidy = TRUE))
data.table::fwrite(data_EW, "RData/EW_query.csv")
} else {
  data_EW <- data.table::fread("RData/EW_query.csv")
}

# -------------- #

### Bring into correct format: EW Age ###

data_EW <- copy(data_EW[, .(areacode = geography_code, 
                            sex      = gender_name,
                            age_code = c_age , 
                            observations = obs_value)])
data_EW[age_code %in% c(117, 118, 119, 120, 6), age_cat :=  "16_24"]
data_EW[age_code %in% c(7, 8), age_cat :=  "25_34"]
data_EW[age_code %in% c(9, 10, 11), age_cat :=  "35_49"]
data_EW[age_code %in% c(12, 13, 14), age_cat :=  "50_64"]
data_EW[age_code %in% c(15, 16), age_cat :=  "65_74"]
data_EW[age_code %in% c(17, 18, 210), age_cat :=  "75p"]

data_EW[, observations_age_cat := sum(observations),
        by = c("areacode", "sex", "age_cat")]

data_EW <- unique(data_EW[, .(areacode, sex, age_cat, observations_age_cat)])
data_EW[, sex := tolower(substr(sex, 1, 1))]
data_EW[, age_sex := paste0(sex,"_",age_cat)]
data_EW <- data_EW[, .(areacode, observations = observations_age_cat, age_sex)]
data_EW <- data.table::dcast(data_EW, areacode ~ age_sex, 
                             value.var = "observations")
data_EW[, total := rowSums(.SD), .SDcols = 2:(dim(data_EW)[2])]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

##### Scotland #####

# identify file paths from here: https://www.nrscotland.gov.uk/statistics-and-data/statistics/statistics-by-theme/population/population-estimates/small-area-population-estimates-2011-data-zone-based/time-series
if (definitions$run_queries == TRUE) {
download.file(url = file.path(paste0("https://www.nrscotland.gov.uk/files//",
          "statistics/population-estimates/sape-time-series/sape-2020.xlsx")), 
              destfile = "RData/sco_2020.xlsx", mode = "wb", timeout = 6000000)
}

# -------------- #

# load file and clean for males 
data_SCO_M <- data.table::data.table(readxl::read_excel(
                        "RData/sco_2020.xlsx", sheet = "Males", skip = 3))
data_SCO_M <- janitor::clean_names(data_SCO_M)
data_SCO_M <- data_SCO_M[, .SD, .SDcols = patterns("data_zone_code|age")]
data_SCO_M <- melt(data_SCO_M, id.vars = "data_zone_code")
data_SCO_M[, age := as.integer(substr(variable, 5, 6))]
data_SCO_M[age <= 15, age_cat := "00_15"]
data_SCO_M[age >= 16 & age <= 24, age_cat := "16_24"]
data_SCO_M[age >= 25 & age <= 34, age_cat := "25_34"]
data_SCO_M[age >= 35 & age <= 49, age_cat := "35_49"]
data_SCO_M[age >= 50 & age <= 64, age_cat := "50_64"]
data_SCO_M[age >= 65 & age <= 74, age_cat := "65_74"]
data_SCO_M[age >= 75 & age, age_cat := "75p"]
data_SCO_M[, observations := sum(value), by = c("age_cat", "data_zone_code")]
data_SCO_M[ , sex := "m"]
data_SCO_M[, age_sex := paste0(sex,"_",age_cat)]
data_SCO_M <- unique(data_SCO_M[age_sex != "m_00_15",
                         .(areacode = data_zone_code, age_sex, observations)])
data_SCO_M <- data.table::dcast(data_SCO_M, areacode ~ age_sex,
                             value.var = "observations")

# -------------- #

# load file and clean for females 
data_SCO_F <- data.table::data.table(readxl::read_excel(
  "RData/sco_2020.xlsx", sheet = "Females", skip = 3))
data_SCO_F <- janitor::clean_names(data_SCO_F)
data_SCO_F <- data_SCO_F[, .SD, .SDcols = patterns("data_zone_code|age")]
data_SCO_F <- melt(data_SCO_F, id.vars = "data_zone_code")
data_SCO_F[, age := as.integer(substr(variable, 5, 6))]
data_SCO_F[age <= 15, age_cat := "00_15"]
data_SCO_F[age >= 16 & age <= 24, age_cat := "16_24"]
data_SCO_F[age >= 25 & age <= 34, age_cat := "25_34"]
data_SCO_F[age >= 35 & age <= 49, age_cat := "35_49"]
data_SCO_F[age >= 50 & age <= 64, age_cat := "50_64"]
data_SCO_F[age >= 65 & age <= 74, age_cat := "65_74"]
data_SCO_F[age >= 75 & age, age_cat := "75p"]
data_SCO_F[, observations := sum(value), by = c("age_cat", "data_zone_code")]
data_SCO_F[, sex := "f"]
data_SCO_F[, age_sex := paste0(sex,"_",age_cat)]
data_SCO_F <- unique(data_SCO_F[age_sex != "f_00_15",
                         .(areacode = data_zone_code, observations, age_sex)])
data_SCO_F <- data.table::dcast(data_SCO_F, areacode ~ age_sex,
                                value.var = "observations", fun.aggregate = NULL)

# -------------- #

# merge males and females 

data_SCO <- merge(data_SCO_M, data_SCO_F, by = "areacode")
data_SCO[, total := rowSums(.SD), .SDcols = 2:(dim(data_SCO)[2])]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Merge GB Nations ###
data_agesex_GB <- rbind(data_EW, data_SCO)
data.table::fwrite(data_agesex_GB, "ROutput/2020_est_age_sex.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

