
# Authors: Kashif Zia & Andreas Hoehn
# Date: 07/12/2023

#===============================Libraries======================================#
library(readr)      # reading data
library(plyr)       # data processing
library(dplyr)      # data manipulation
library(tidyverse)  # data manipulation
#===============================================================================

# ============================== FETCH ALL DATA ============================== #
filepath <- paste0(getwd(),"/RData/")
filenamei <- "_indresp.tab" # file name US individuals 
filenameh <- "_hhresp.tab" # filename US households (needed for cross-tabulation)
w <- "k" # wave k; change it to work with any other wave

# full filenamei read; individual adult data
# full filenameh read; individual household data
ind_full <- read.csv(paste(filepath, w, filenamei, sep=""), sep = "\t", header = T) 
hh_full <-  read.csv(paste(filepath, w, filenameh, sep=""), sep = "\t", header = T)
# file path is concatenated with file name and wave name
# csv is read separated by tab character

# cross-tabulation, Join based on hhid, select hhid and tenure and add it with the original dataset 
ind_full <- ind_full %>%
  inner_join(hh_full %>% select(k_hidp, k_tenure_dv), by = "k_hidp")

# select attributes
ind_full <-
  ind_full %>% select(
    pidp, # personal identifier
    k_hidp, # household identifier
    k_sex, # person's sex
    k_age_dv, # person's age category
    k_hiqual_dv, # person's highest qualification category
    k_racel_dv, # person's race category
    k_marstat, # person's marital status
    k_jbstat, # person's job status
    k_health, # is there a health condition?
    k_hhtype_dv, # person's household type
    k_tenure_dv, # person's household tenure
    k_gor_dv, # larger region in which the person is residing
    k_scsf1 # general health condition
    # Add more attributes here if needed
  )
# view some raw individual adult data
head(ind_full[c(1:13)],9)

# Renaming column names; ending with 0 indicate the original values 
colnames(ind_full) <-
  c(
    "pidp",
    "hidp",
    "sex0",
    "age0",
    "hiqual0",
    "ethnicity0",
    "marstat0",
    "jbstat0",
    "health0",
    "hhtype0",
    "htenure0",
    "gov0",
    "scsf10"
    # add more labels if more attributes are added above
  )
# view some raw individual adult data
head(ind_full[c(1:13)],9)

# exclude NI (GB)
ind_GB <- ind_full[!(ind_full$gov0==12),]
#===============================================================================


# ==================== DATA PROCESSING (match constraints) =================== #
ind <- ind_GB # just a copy

# ************ age and sex 

# Variables = k_sex, k_age_dv Variable label = Sex and Age
# Either use Sex and Age separately or Sex_Age combination
# All these combinations are ready to be used with matching constraints.
ind <- ind %>% mutate(sex = case_when(sex0 == 1 ~ "M", sex0 == 2 ~ "F"))
ind <- ind %>% mutate(age = case_when(age0 != -9 ~ age0))
ind <- ind %>% drop_na(sex, age) # Drop if have value NA, both sex and age

# breaks and labels 
brks <- c(15, 24, 34, 49, 64, 74, Inf)
labs <- c("16_24", "25_34", "35_49", "50_64", "65_74", "75_up")
# Categorise age.
ind$age <- cut(ind$age, breaks = brks, labels = labs) 
# Generate the sex & age category.
ind$sex_age <-
  as.character(paste(ind$sex, ind$age, sep = "_")) 

# ************ Highest Qualification

# Variable = k_hiqual_dv	Variable label = Highest qualification
#	Value = 1.0	Label = Degree
#	Value = 2.0	Label = Other higher degree
#	Value = 3.0	Label = A-level etc
#	Value = 4.0	Label = GCSE etc
#	Value = 5.0	Label = Other qualification
# Value = 9.0	Label = No qualification
#	Value = -1.0	Label = don't know
#	Value = -9.0	Label = missing
#	Value = -8.0	Label = inapplicable
#	Value = -2.0	Label = refusal

# A level of inconsistencies requiring simplification
# Scotland even has less number of level; apprenticeship not there
# So we needed to simplify so that everything fits in, that is

ind <-
  ind %>% mutate(
    hiqual = case_when(
      hiqual0 == 1 ~ "level_4_and_above",
      hiqual0 == 2 ~ "level_4_and_above",
      hiqual0 == 3 ~ "level_3",
      hiqual0 == 4 ~ "level_1_to_2",
      hiqual0 == 5 ~ "level_1_to_2", # etc means apprenticeships etc. and others
      hiqual0 == 9 ~ "none"
      # others automatically become NA, that will be dropped at the end 
    )
  )

# ************ ethnicity

# Variable = k_racel_dv	Variable label = Ethnic group 

#	Value = 1.0	Label = british/english/scottish/welsh/northern irish (white)
#	Value = 2.0	Label = irish (white)
#	Value = 3.0	Label = gypsy or irish traveller (white)
#	Value = 4.0	Label = any other white background (white)
#	Value = 5.0	Label = white and black caribbean (mixed)
#	Value = 6.0	Label = white and black african (mixed)
#	Value = 7.0	Label = white and asian (mixed)
#	Value = 8.0	Label = any other mixed background (mixed)
#	Value = 9.0	Label = indian (asian or asian british)
#	Value = 10.0	Label = pakistani (asian or asian british)
#	Value = 11.0	Label = bangladeshi (asian or asian british)
#	Value = 12.0	Label = chinese (asian or asian british)
#	Value = 13.0	Label = any other asian background (asian or asian british)
#	Value = 14.0	Label = caribbean (black or black british)
#	Value = 15.0	Label = african (black or black britih)
#	Value = 16.0	Label = any other black background (black or black britih)
#	Value = 17.0	Label = arab (other ethnic group)
#	Value = 97.0	Label = any other ethnic group (other ethnic group)
#	Value = -1.0	Label = don't know
#	Value = -9.0	Label = missing
#	Value = -8.0	Label = inapplicable
#	Value = -2.0	Label = refusal

# have to combine to generate 5 high level categories
ind <-
  ind %>% mutate(
    ethnicity = case_when(
      ethnicity0 >= 1 &
        ethnicity0 <= 4 ~ "white",
      ethnicity0 >= 5 &
        ethnicity0 <= 8 ~ "mixed",
      ethnicity0 >= 9 &
        ethnicity0 <= 13 ~ "asian",
      ethnicity0 >= 14 &
        ethnicity0 <= 16 ~ "black",
      ethnicity0 == 17 |
        ethnicity0 == 97 ~ "other"
      # others automatically become NA, that will be dropped at the end
    )
  )

# ************ marital status

#	Variable = k_marstat	Variable label = Legal marital status
#	Value = 1.0	Label = Single, nvr marr/civ p
#	Value = 2.0	Label = Married
#	Value = 3.0	Label = Civil Partner (legal)
#	Value = 4.0	Label = Separated legally marr
#	Value = 5.0	Label = Divorced
#	Value = 6.0	Label = Widowed
#	Value = 7.0	Label = Sep from Civil Partner
#	Value = 8.0	Label = A former Civil Partner
#	Value = 9.0	Label = Surviving Civil Partner
#	Value = -2.0	Label = refusal
#	Value = -9.0	Label = missing
#	Value = -8.0	Label = inapplicable
#	Value = -7.0	Label = proxy
#	Value = -1.0	Label = don't know

# have to combine to generate the following categories
ind <-
  ind %>% mutate(
    marstat = case_when(
      marstat0 == 1 ~ "single",
      marstat0 == 2 ~ "married",
      marstat0 == 3 ~ "civil_partner",
      marstat0 == 4 ~ "separated",
      marstat0 == 5 ~ "divorced",
      marstat0 == 6 ~ "widowed_surviving",
      marstat0 == 7 ~ "separated",
      marstat0 == 8 ~ "divorced",
      marstat0 == 9 ~ "widowed_surviving"
      # others automatically become NA, that will be dropped at the end
    )
  )

# ************ Job Status

#Variable = k_jbstat Varibale label = jon status
#Value = 1.0	Label = Self employed
#Value = 2.0	Label = Paid employment(ft/pt)
#Value = 3.0	Label = Unemployed
#Value = 4.0	Label = Retired
#Value = 5.0	Label = On maternity leave
#Value = 6.0	Label = Family care or home
#Value = 7.0	Label = Full-time student
#Value = 8.0	Label = LT sick or disabled
#Value = 9.0	Label = Govt training scheme
#Value = 10.0	Label = Unpaid, family business
#Value = 11.0	Label = On apprenticeship
#Value = 12.0	Label = On furlough
#Value = 13.0	Label = Temporarily laid off/short term working
#Value = 97.0	Label = Doing something else
#Value = -2.0	Label = refusal
#Value = -9.0	Label = missing
#Value = -8.0	Label = inapplicable
#Value = -1.0	Label = don't know

# have to combine to generate the following categories
ind <-
  ind %>% mutate(
    jbstat = case_when(
      jbstat0 == 1 ~ "self_employed",
      jbstat0 == 2 ~ "in_paid_employment",
      jbstat0 == 3 ~ "unemployed",
      jbstat0 == 4 ~ "retired",
      jbstat0 == 5 ~ "other",
      jbstat0 == 6 ~ "looking_after_home",
      jbstat0 == 7 ~ "student",
      jbstat0 == 8 ~ "long_term_sick_disabled",
      jbstat0 == 9 |
        jbstat0 == 10 | jbstat0 == 11 | jbstat0 == 12 | jbstat0 == 13 | jbstat0 == 97 ~ "other",
      #others automatically become NA, that will be dropped at the end 
    )
  )

# ************ household composition

#Variable = k_hhtype_dv	Variable label = Composition of household, LFS-version
#	Value = 1.0	Label = 1 male, aged 65+, no children
#	Value = 2.0	Label = 1 female, age 60+, no children
#	Value = 3.0	Label = 1 adult under pensionable age, no children
#	Value = 4.0	Label = 1 adult, 1 child
#	Value = 5.0	Label = 1 adult, 2 or more children
#	Value = 6.0	Label = Couple both under pensionable age, no children
#	Value = 8.0	Label = Couple 1 or more over pensionable age,no children
#	Value = 10.0	Label = Couple with 1 child
#	Value = 11.0	Label = Couple with 2 children
#	Value = 12.0	Label = Couple with 3 or more children
#	Value = 16.0	Label = 2 adults, not a couple, both under pensionable age, no children
#	Value = 17.0	Label = 2 adults, not a couple, one or more over pensionable age, no children
#	Value = 18.0	Label = 2 adults, not a couple, 1 or more children
#	Value = 19.0	Label = 3 or more adults, no children, incl. at least one couple
#	Value = 20.0	Label = 3 or more adults, 1-2 children, incl. at least one couple
#	Value = 21.0	Label = 3 or more adults, >2 children, incl. at least one couple
#	Value = 22.0	Label = 3 or more adults, no children, excl. any couples
#	Value = 23.0	Label = 3 or more adults, 1 or more children, excl. any couples
#	Value = -1.0	Label = don't know
#	Value = -9.0	Label = missing
#	Value = -8.0	Label = inapplicable
#	Value = -2.0	Label = refusal

# have to combine to generate the following categories
ind <-
  ind %>% mutate(
    hhtype = case_when(
      hhtype0 == 1 ~ "one_person_no_child",
      hhtype0 == 2 ~ "one_person_no_child",
      hhtype0 == 3 ~ "one_person_no_child",
      hhtype0 == 4 ~ "one_person_with_children",
      hhtype0 == 5 ~ "one_person_with_children",
      hhtype0 == 6 ~ "married_no_child",
      hhtype0 == 8 ~ "married_no_child",
      hhtype0 == 10 ~ "married_with_children",
      hhtype0 == 11 ~ "married_with_children",
      hhtype0 == 12 ~ "married_with_children",
      hhtype0 == 16 ~ "cohabiting_no_child",
      hhtype0 == 17 ~ "cohabiting_no_child",
      hhtype0 == 18 ~ "cohabiting_with_children",
      hhtype0 == 19 ~ "other_family_no_child",
      hhtype0 == 20 ~ "other_not_family_with_children",
      hhtype0 == 21 ~ "other_not_family_with_children",
      hhtype0 == 22 ~ "other_not_family_no_child",
      hhtype0 == 23 ~ "other_not_family_with_children"
      #others automatically become NA, that will be dropped at the end 
    )
  )

# ************ health  
#Variable = k_scsf1	Variable label = General health
# Value = 1.0	Label = Excellent
# Value = 2.0	Label = Very good
# Value = 3.0	Label = Good
# Value = 4.0	Label = Fair
# Value = 5.0	Label = Poor
# Value = -1.0	Label = don't know
# Value = -9.0	Label = missing
# Value = -8.0	Label = inapplicable
# Value = -7.0	Label = proxy
# Value = -2.0	Label = refusal

# have to combine to generate the following categories
ind <-
  ind %>% mutate(
    scsf1 = case_when(
      scsf10 == 1 ~ "Very good",
      scsf10 == 2 ~ "Very good",
      scsf10 == 3 ~ "Good",
      scsf10 == 4 ~ "Fair",
      scsf10 == 5 ~ "Poor"
      #others automatically become NA, that will be dropped at the end 
    )
  )

# ************ household (from hh data) tenure

#Variable = k_tenure_dv	Variable label = housing tenure
#	Value = 1.0	Label = Owned outright
#	Value = 2.0	Label = Owned with mortgage
#	Value = 3.0	Label = Local authority rent
#	Value = 4.0	Label = Housing assoc rented
#	Value = 5.0	Label = Rented from employer
#	Value = 6.0	Label = Rented private unfurnished
#	Value = 7.0	Label = Rented private furnished
#	Value = 8.0	Label = Other
#	Value = -2.0	Label = refusal
#	Value = -9.0	Label = missing
#	Value = -8.0	Label = inapplicable
#	Value = -1.0	Label = don't know

# have to combine to generate the following categories
ind <-
  ind %>% mutate(
    htenure = case_when(
      htenure0 == 1 ~ "owned_outright",
      htenure0 == 2 ~ "owned_mortgage",
      htenure0 == 3 ~ "social_rented",
      htenure0 == 4 ~ "social_rented",
      htenure0 == 5 ~ "private_rented",
      htenure0 == 6 ~ "private_rented",
      htenure0 == 7 ~ "private_rented",
      htenure0 == 8 ~ "private_rented" # Private_rented? 
      #others automatically become NA, that will be dropped at the end 
    )
  )



# Drop individuals with missing socioeconomic characteristics.
ind <-
  ind %>% drop_na(hiqual, ethnicity, marstat, jbstat, hhtype, htenure, scsf1)
ind <- ind %>% arrange(pidp)
# view some raw individual adult data
head(ind[c(1:22)],9)
paste("The number of rows reduced from ", nrow(ind_full), " to ", nrow(ind))
#===============================================================================


#============================WRITE TO FILE======================================
# Write to file
file_out <- paste0(getwd(),"/ROutput/","k_indresp_hhresp_modified.csv")
write.csv(ind, file = file_out, row.names=F) #Write ind data for wave k
#===============================================================================

