### Meta ###

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2024-06-07
# About: this file loads constraint data, lookup, and builds the SP

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Load Underlying Constraints Data ### 

# load data on constraints 
cons_sexage    <- data.table::fread("RData/constraints/2020_est_age_sex.csv")
cons_marstat   <- data.table::fread("RData/constraints/2011_census_marital.csv")
cons_hiqualif  <- data.table::fread("RData/constraints/2011_census_qualification.csv")
cons_hhtype    <- data.table::fread("RData/constraints/2011_census_hh_composition.csv")
cons_hhtenure  <- data.table::fread("RData/constraints/2011_census_hh_tenure.csv")
cons_ethnicity <- data.table::fread("RData/constraints/2011_census_ethnicity.csv")
cons_ecostatus <- data.table::fread("RData/constraints/2011_census_economic.csv")
cons_genhealth <- data.table::fread("RData/constraints/2011_census_general_health.csv")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Load Geo Lookups ###

# Load Geo Lookup: census 2011
lookup_oa <- data.table::fread("RData/lookup/OA_to_Local_Authority_District_May_2021.csv")
lookup_oa <- unique(lookup_oa[, .(lsoa11cd, msoa11cd, ladcd, ladnm)])
lookup_oa[, ctr_code := substr(lsoa11cd, 1, 1)]

# subset for geographies covered in definitions
lookup_oa <- lookup_oa[ctr_code %in% definitions$geo_subset, ]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Load LSOA Shapefile for LSOA Resolution ###

# load
shapefile <- readOGR(paste0("RData/lookup/",
                            "LSOA_2011_EW_BGC_V3.shp"))

# subset for geographies covered in definitions
shapefile <- subset(shapefile, LSOA11CD %in% 
          unlist(lookup_oa[ladnm %in% definitions$geo_example, .(lsoa11cd)]))

# indicate region variable in shapefile
shapefile <- fortify(shapefile, region = "LSOA11CD")

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #



### Load SP Backbone: With Health ###

# load sp # 
sp_8cons <- data.table::fread("RData/sp_final/sp_ind_wavek_census2011_est2020_8cons.csv")
setkey(sp_8cons, synthetic_zone, pidp)

# subset for geography of interest
sp_8cons[, ctr_code := substr(synthetic_zone, 1, 1)]
sp_8cons <- sp_8cons[ctr_code %in% definitions$geo_subset, ]

# test for clones
test <- table(sp_8cons$pidp)
test2 <- data.table(test)
setkey(test2, N)
mean(test2$N)
min(test2$N)
max(test2$N)
test4 <- sp_8cons[, .(area_clones = .N), by = c("pidp", "synthetic_zone")]
test4 <- test4[, .(unique_areas = length(unique(synthetic_zone))), by = c("pidp")]
setkey(test4, unique_areas)
min(test4$unique_areas)
mean(test4$unique_areas)
max(test4$unique_areas)


# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Load US raw data for individuals ###

# load us hh data # 
us_ind <- data.table::fread("RData/us_raw/k_indresp.tab")

# rename # 
data.table::setnames(us_ind, old = names(us_ind),
        new = gsub(paste0("^",definitions$us_wave,"_"),"",names(us_ind)))

# define variables to keep
us_ind_keep_var <- c("pidp", "hidp", "indpxui_xw",        # ID & weight
  "sex", "age_dv",	"hiqual_dv", "racel_dv",	"marstat",  # constraints 1
  "jbstat",	"hhtype_dv", "scsf1",                         # constraints 2
  "sf12mcs_dv", "sf12pcs_dv",                             # for health comparison
  "jbnssec5_dv",                                          # for NS-SEC comparison 
  "bendis1", "bendis2", "bendis3", "benbase2", "benbase4")# for employment deprivation count                                                        
          
us_ind <- us_ind[, ..us_ind_keep_var]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Load US raw data for households ###

# load us hh data # 
us_hh <- data.table::fread("RData/us_raw/k_hhresp.tab")

# rename # 
data.table::setnames(us_hh, old = names(us_hh),
                     new = gsub(paste0("^",definitions$us_wave,"_"),"",names(us_hh)))

# define variables to keep
us_hh_keep_var <- c("hidp", "tenure_dv")

us_hh <- us_hh[, ..us_hh_keep_var]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Merge US data for individuals and hh ###

us_both <- merge(us_ind, us_hh, by = c("hidp"), all.x = TRUE)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### Create SP with all Constraint Variables: 8 constraint SPs ###

sp_8cons <- merge(sp_8cons, us_both, by = c("pidp"), all.x = TRUE)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### External Validation: Load SIMD Rank Data ###

# website: https://www.gov.scot/publications/scottish-index-of-multiple-deprivation-2020v2-ranks/

# run query
if (definitions$run_queries == TRUE) {
  download.file(url = file.path(paste0("https://www.gov.scot/binaries/content/",
    "documents/govscot/publications/statistics/2020/01/scottish-index-of-",
    "multiple-deprivation-2020-ranks-and-domain-ranks/documents/scottish-index-of-",
    "multiple-deprivation-2020-ranks-and-domain-ranks/scottish-index-of-multiple-",
    "deprivation-2020-ranks-and-domain-ranks/govscot%3Adocument/SIMD%2B2020v2%2B-%2Branks.xlsx")), 
        destfile = "RData/external_validation/simd_ranks.xlsx", mode = "wb",
        timeout = definitions$timeout)
} else {print("no download, instead load already downloaded data")}

# load file
data_simd_ranks <- data.table::data.table(readxl::read_xlsx(
  "RData/external_validation/simd_ranks.xlsx", sheet = "SIMD 2020v2 ranks", col_names = TRUE))

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### External Validation: Load SIMD Absolute Indicator Data ###

# website: https://www.gov.scot/publications/scottish-index-of-multiple-deprivation-2020v2-indicator-data/

if (definitions$run_queries == TRUE) {
  download.file(url = file.path(paste0("https://www.gov.scot/binaries/content/",
    "documents/govscot/publications/statistics/2020/01/scottish-index-of-",
    "multiple-deprivation-2020-indicator-data/documents/simd_2020_indicators/",
    "simd_2020_indicators/govscot%3Adocument/SIMD%2B2020v2%2B-%2Bindicators.xlsx")), 
      destfile = "RData/external_validation/simd_indicators.xlsx", mode = "wb",
      timeout = definitions$timeout)
} else {print("no download, instead load already downloaded data")}

# load file
data_simd_indicators <- data.table::data.table(readxl::read_xlsx(
  "RData/external_validation/simd_indicators.xlsx", sheet = "Data", col_names = TRUE))

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### External Validation: Load SCO Census 2011 Data for NS-SeC classification  ###


# website: https://www.scotlandscensus.gov.uk/documents/2011-census-table-data-sns-data-zone-2011/

if (definitions$run_queries == TRUE) {
  download.file(url = file.path(paste0("https://nrscensusprodumb.blob.core.windows.net/",
                                       "downloads/SNS%20Data%20Zone%202011%20blk.zip")), 
                destfile = "RData/external_validation/sco_2011.zip", mode = "wb",
                timeout = definitions$timeout)
  # unzip file 
  unzip(zipfile = "RData/external_validation/sco_2011.zip",
        exdir = "RData/external_validation/")
} else {print("no download, instead load already downloaded data")}

# load file
data_nssec_sco <- data.table::fread("RData/external_validation/LC6114SC.csv",
                                    header = TRUE)
data.table::setnames(data_nssec_sco, old = names(data_nssec_sco),
        new = c("areacode", "age", "total",
                "V1", "V2", "V3", "V4", "V5", "V6", "V7", "V8", "students"))
data_nssec_sco <- data_nssec_sco[!(areacode %in% c("S92000003","")) & 
                      age %in% c("All people aged 16 to 74"),
                      .(areacode, V1, V2, V3, V4, V5, V6, V7, V8, students)]

data_nssec_sco[, census_management_professional := V1 + V2]
data_nssec_sco[, census_intermediate := V3]
data_nssec_sco[, census_small_employers_own_account := V4]
data_nssec_sco[, census_lower_supervisory_technical := V5]
data_nssec_sco[, census_semi_routine_routine := V6 + V7]
data_nssec_sco[, census_other := V8 + students]
data_nssec_sco <-  data_nssec_sco[,
      .(areacode, census_management_professional, census_intermediate, 
      census_small_employers_own_account, census_lower_supervisory_technical,
      census_semi_routine_routine, census_other)]

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #
