### Meta ###

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2024-06-07
# About: this file provides some examples of the internal validation 
# (SP vs. lookup & constraints etc.)

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### [1] Do all areas from the official 2011 census lookup appear in the SP?
# identify LSOAs not in SP based on 2011 census geography lookup ###
diagnostics$unmatched_areas_lookup <- lookup_oa[!(lsoa11cd %in% unique(sp_8cons$synthetic_zone)), ]
# check by hand using SIMD online tool AND constraints: 
# S01010226: pop size of 0 in SIMD and 2020 Pop Estimates, but not 0 in 2011 census
# S01010227: pop size of 0 in SIMD and 2020 Pop Estimates, but not 0 in 2011 census
# S01010206: pop size of 0 in SIMD and 2020 Pop Estimates, but not 0 in 2011 census

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### [2] Does the number of individuals in constraint files matches obs in SP? ### 

# merge all constraint files 
N_ind_8cons <- merge(cons_sexage[, .(areacode , N_sexage = total)],
                     cons_marstat[, .(areacode , N_marstat = total)],
                     by = "areacode",  all.x = TRUE)
N_ind_8cons <- merge(N_ind_8cons,
                     cons_hiqualif[, .(areacode , N_hiqualif = total)],
                     by = "areacode")
N_ind_8cons <- merge(N_ind_8cons,
                     cons_hhtype[, .(areacode , N_hhtype  = total)],
                     by = "areacode")
N_ind_8cons <- merge(N_ind_8cons,
                     cons_hhtenure[, .(areacode , N_hhtenure  = total)],
                     by = "areacode")
N_ind_8cons <- merge(N_ind_8cons,
                     cons_ethnicity[, .(areacode , N_ethnicity  = total)],
                     by = "areacode")
N_ind_8cons <- merge(N_ind_8cons,
                     cons_ecostatus[, .(areacode , N_ecostatus  = total)],
                     by = "areacode")
N_ind_8cons <- merge(N_ind_8cons,
                     cons_genhealth[, .(areacode , N_genhealth  = total)],
                     by = "areacode")

# ----------- #

# merge all constraints with information from sp_8cons 
cons_sp_8cons <- sp_8cons[, .(total = .N), by = "synthetic_zone"]
N_ind_8cons <- merge(N_ind_8cons,
                     cons_sp_8cons[, .(areacode  = synthetic_zone, N_sp_8cons  = total)],
                     by = "areacode")

# create difference in number ob observations between sp_8cons and constraints 
N_ind_8cons[, diff_sexage := N_sp_8cons - N_sexage][,
              diff_marstat := N_sp_8cons - N_marstat][,
              diff_hiqualif := N_sp_8cons - N_hiqualif][,
              diff_hhtype := N_sp_8cons - N_hhtype][,
              diff_hhtenure := N_sp_8cons - N_hhtenure][,
              diff_ethnicity := N_sp_8cons - N_ethnicity][,
              diff_ecostatus:= N_sp_8cons - N_ecostatus][,
              diff_genhealth:= N_sp_8cons - N_genhealth]

# summaries
diagnostics$N_total_constraints <- data.table::data.table(
  rbind(summary(N_ind_8cons$diff_sexage),
        summary(N_ind_8cons$diff_marstat),
        summary(N_ind_8cons$diff_hiqualif),
        summary(N_ind_8cons$diff_hhtype),
        summary(N_ind_8cons$diff_hhtenure),
        summary(N_ind_8cons$diff_ethnicity),
        summary(N_ind_8cons$diff_ecostatus),
        summary(N_ind_8cons$diff_genhealth)))
diagnostics$N_total_constraints[, constraint_name :=
            c("Sex Age","Marital Status", "Highest Qualification",
              "HH Type", "HH Tenure", "Ethnicity", "Economic Activity", 
              "General Health")]
data.table::setcolorder(diagnostics$N_total_constraints, c("constraint_name"))

# translate to long format for plotting 
N_ind_8cons <- N_ind_8cons[, .(areacode , diff_sexage, diff_marstat,
                               diff_hiqualif, diff_hhtype, diff_hhtenure,
                               diff_ethnicity, diff_ecostatus, diff_genhealth)]
N_ind_8cons <- melt(N_ind_8cons, 
                    measure.vars = c("diff_sexage", "diff_marstat",
                                     "diff_hiqualif", "diff_hhtype",
                                     "diff_hhtenure", "diff_ethnicity",
                                     "diff_ecostatus", "diff_genhealth"),
                    variable.name = "indicator", value.name = "difference")

# rename indicators for plot 
N_ind_8cons[
  indicator == "diff_sexage", indicator := "SP vs. sex and age (master constraint)"][
  indicator == "diff_marstat", indicator := "SP vs. marital status constraint"][
  indicator == "diff_hiqualif", indicator := "SP vs. qualification constraint"][
  indicator == "diff_hhtype", indicator := "SP vs. HH type constraint"][
  indicator == "diff_hhtenure", indicator := "SP vs. HH tenure constraint"][
  indicator == "diff_ethnicity", indicator := "SP vs. ethnicity constraint"][
  indicator == "diff_ecostatus", indicator := "SP vs. economic status constraint"][
  indicator == "diff_genhealth", indicator := "SP vs. general health constraint"]

# Country Code 
N_ind_8cons[, ctr_code := substr(areacode , 1, 1)][
  ctr_code == "E", ctr_code := "England"][
  ctr_code == "S", ctr_code := "Scotland"][
  ctr_code == "W", ctr_code := "Wales"]

# visualise differences 
N_ind_8cons_plot <- ggplot(N_ind_8cons[indicator != "SP vs. sex and age (master constraint)"],
                           aes(x = difference, fill = ctr_code, colour = ctr_code)) +
  annotate("rect", xmin = 0, xmax = Inf, ymin = -Inf, ymax = Inf,
           alpha = definitions$sep_alpha, fill = definitions$sep_cols[1]) + 
  annotate("rect", xmin = -Inf, xmax = 0, ymin = -Inf, ymax = Inf,
           alpha = definitions$sep_alpha, fill = definitions$sep_cols[2]) +
  geom_histogram(binwidth = 1) +
  labs(fill = "", colour = "",
       x = "difference in number of individuals per LSOA (England and Wales) / DZ (Scotland)",
       y = "number of LSOAs (England and Wales) / DZ (Scotland)",
       caption = paste0("note: age and sex constraint not shown as this is the master constraint with zero difference across all areas")) +
  xlim(-3000,3000) +
  facet_wrap(~ indicator, ncol = 3, scales = "fixed") + 
  geom_vline(xintercept = 0, linewidth = 0.25, col = definitions$supp_col) + 
  theme_classic() + 
  geom_text(x =  1500, y = 200,  label = "SP overestimates", size = 2, col = definitions$supp_col) + 
  geom_text(x = -1500, y = 200, label = "SP underestimates", size = 2, col = definitions$supp_col)
ggplot2::ggsave(N_ind_8cons_plot, filename = "ROutput/N_ind_8cons_plot.jpeg",
                units = "cm", width =  definitions$fig_width, height = definitions$fig_height)  

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #
