### Meta ###

# Author: Andreas Hoehn
# Version: 1.0
# Date:  2024-06-07
# About: this file performs an external validation exercise for Scotland using 
# the data zone level. We perform a validation using the following three examples:
# (1) corr. between rank in average sf12 pcs mcs scores and SIMD rank for health domain
# (2) corr. between number of employment-deprived individuals in SP and SIMD data (2017 DWP estimates)
# (3) overlap of within DZ numbers for a constraint-related feature such as NS-Sec 
# in comparison with census 2011 estimates

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### [1] Correlation with SIMD Health Domain ###

# subset SIMD rank data # 
data_simd_health <- data_simd_ranks[, .SD,
  .SDcols = names(data_simd_ranks) %like% "Data_Zone|SIMD2020_Health_Domain_Rank"]

# rank sp 7 cons areas by sf12  
simd_comp_8cons <- sp_8cons[ctr_code == "S",
                      .(sf12mcs_8cons = mean(sf12mcs_dv),
                        sf12pcs_8cons = mean(sf12pcs_dv),
                        sf12both_8cons =  (mean(sf12mcs_dv) + mean(sf12pcs_dv)) / 2),
                            by = c("synthetic_zone")]
setkey(simd_comp_8cons, sf12mcs_8cons)
simd_comp_8cons[, sf12mcs_8cons_rank := .I]
setkey(simd_comp_8cons, sf12pcs_8cons)
simd_comp_8cons[, sf12pcs_8cons_rank := .I]
setkey(simd_comp_8cons, sf12both_8cons)
simd_comp_8cons[, sf12both_8cons_rank := .I]

# melt
data_simd_health_comp <- melt(
  simd_comp_8cons[, .SD, .SDcols = names(simd_comp_8cons) %like% "synthetic_zone|rank"],
  id.var = c("synthetic_zone"))

# merge 
data_simd_health_comp <- merge(data_simd_health_comp, data_simd_health,
                   by.x = "synthetic_zone", by.y = "Data_Zone")

# sf12 both 8 cons 
corr_sf12both_8cons <- round(cor.test(
  x = unlist(data_simd_health_comp[variable == "sf12both_8cons_rank", 
                            .(SIMD2020_Health_Domain_Rank)]),
  y = unlist(data_simd_health_comp[variable == "sf12both_8cons_rank", 
                            .(value)]), method = 'spearman')$estimate,
  definitions$round)

# sf12 pcs 8 cons 
corr_sf12pcs_8cons <- round(cor.test(
  x = unlist(data_simd_health_comp[variable == "sf12pcs_8cons_rank", 
                            .(SIMD2020_Health_Domain_Rank)]),
  y = unlist(data_simd_health_comp[variable == "sf12pcs_8cons_rank", 
                            .(value)]), method = 'spearman')$estimate,
  definitions$round)

# sf12 mcs 8 cons 
corr_sf12mcs_8cons <- round(cor.test(
  x = unlist(data_simd_health_comp[variable == "sf12mcs_8cons_rank", 
                            .(SIMD2020_Health_Domain_Rank)]),
  y = unlist(data_simd_health_comp[variable == "sf12mcs_8cons_rank", 
                            .(value)]), method = 'spearman')$estimate,
  definitions$round)

# plot
simd_correlation_plot <- ggplot(data_simd_health_comp,
                                aes(x = SIMD2020_Health_Domain_Rank, y = value)) + 
  geom_point(size = 0.5, alpha = definitions$dot_alpha) + 
  lims(x= c(0, 7500), y = c(0, 7500)) +
  geom_smooth(method = "lm", se=FALSE, color = definitions$fit_col, formula = y ~ x) +
  theme_classic() +
  labs(x = "rank of DZ in SIMD 2020",
       y = "rank of DZ in SIPHER's Synthetic Population") +
  facet_wrap( ~ variable, ncol = 2, 
              labeller = labeller(variable = 
              c("sf12mcs_8cons_rank"  = "average SF-12 MCS scores",
                "sf12pcs_8cons_rank"  = "average SF-12 PCS scores",
                "sf12both_8cons_rank" = "combined average SF-12 PCS and MCS scores"))) + 
  geom_ribbon(stat = 'function', fun = .SPlitPlot45,
              mapping = aes(ymin = after_stat(y), ymax = Inf),
              fill = definitions$sep_cols[1], alpha = definitions$sep_alpha) +
  geom_ribbon(stat = 'function', fun = .SPlitPlot45,
              mapping = aes(ymin = after_stat(0), ymax = after_stat(y)),
              fill = definitions$sep_cols[2], alpha = definitions$sep_alpha) +
  coord_cartesian(expand = FALSE) + 
  geom_text(x = 1000, y =  6200, label = "SP overestimates", size = 3) + 
  geom_text(x = 6200, y =  800, label = "SP underestimates", size = 3)
ggplot2::ggsave(simd_correlation_plot,
                filename = "ROutput/simd_health_correlation.jpeg", units = "cm", 
                width =  definitions$fig_width, height = definitions$fig_height)  

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### [2] Prediction of Employment Deprivation Prevalence ### 

# initialise employment deprivation
sp_8cons_working_age_SC <- sp_8cons[ctr_code %in% c("S") & age_dv >= 16 & age_dv <= 65,] 
sp_8cons_working_age_SC[, employment_deprived := 0]
sp_8cons_working_age_SC[bendis1 == 1 | bendis2 == 1 | bendis3 == 1 | benbase2 == 1 |
                          (benbase4 == 1 & jbstat == 3), employment_deprived := 1]

sp_8cons_working_age_SC <-sp_8cons_working_age_SC[,
            .(sp_employment_count = sum(employment_deprived),
              sp_employment_rate = sum(employment_deprived) / .N), by = "synthetic_zone"]

# subset loaded SIMD indicator ddata
data_simd_employment <- data_simd_indicators[,
                  .(Data_Zone, Employment_count, Employment_rate)]
data.table::setnames(data_simd_employment, old = names(data_simd_employment),
                     new = c("synthetic_zone", "simd_employment_count", "simd_employment_rate"))

sp_8cons_working_age_SC <- merge(sp_8cons_working_age_SC, 
                                 data_simd_employment, by= "synthetic_zone")

cor(sp_8cons_working_age_SC$simd_employment_count,
    sp_8cons_working_age_SC$sp_employment_count)

# plot
simd_employment_plot <- ggplot(sp_8cons_working_age_SC,
                           aes(x = simd_employment_count, y = sp_employment_count)) + 
  geom_point(size = 0.5, alpha = definitions$dot_alpha) + 
  lims(x= c(0, 325), y = c(0, 325)) +
  labs(x = "number of employment-deprived individuals based on SIMD 2020, per DZ",
       y = "number of employment-deprived individuals in SIPHER's SP, per DZ") +
  #geom_abline(intercept = 0, slope = 1, linewidth = 0.1) +
  theme_classic() + 
  geom_text(x = 185, y = 25,  label = "SP underestimates", size = 8) + 
  geom_text(x = 185, y = 250, label = "SP overestimates", size = 8) +
  geom_function(fun = .SPlitPlot45) +
  geom_smooth(method = "lm", se = FALSE, color = definitions$fit_col, formula = y ~ x, linewidth = 1) +
  geom_ribbon(stat = "function", fun = .SPlitPlot45,
              mapping = aes(ymin = after_stat(y), ymax = Inf),
              fill = definitions$sep_cols[1], alpha = definitions$sep_alpha) +
  geom_ribbon(stat = "function", fun = .SPlitPlot45,
              mapping = aes(ymin = after_stat(0), ymax = after_stat(y)),
              fill = definitions$sep_cols[2], alpha = definitions$sep_alpha) +
  coord_cartesian(expand = FALSE)
ggplot2::ggsave(simd_employment_plot, filename = "ROutput/simd_employment_plot.jpeg",
                units = "cm", width =  definitions$fig_width, height = definitions$fig_height)          

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #

### [3] Prediction of Ns-Sec Codes ###

### NS-SeC 5 Category from SP with 8 cons ###
sp_8cons_nssec5_SC <- sp_8cons[ctr_code %in% c("S") & age_dv >= 16 & age_dv <= 74,
                               .(synthetic_zone, nssec_sp = jbnssec5_dv)]
sp_8cons_nssec5_SC[, nssec_sp := as.character(nssec_sp)]
sp_8cons_nssec5_SC[nssec_sp %in% c("1"), nssec_sp := "sp_management_professional"]
sp_8cons_nssec5_SC[nssec_sp %in% c("2"), nssec_sp := "sp_intermediate"]
sp_8cons_nssec5_SC[nssec_sp %in% c("3"), nssec_sp := "sp_small_employers_own_account"]
sp_8cons_nssec5_SC[nssec_sp %in% c("4"), nssec_sp := "sp_lower_supervisory_technical"]
sp_8cons_nssec5_SC[nssec_sp %in% c("5"), nssec_sp := "sp_semi_routine_routine"]
sp_8cons_nssec5_SC[nssec_sp %in% c("-1","-9","-8","-7","-2"), nssec_sp := "sp_other"]
sp_8cons_nssec5_SC <- sp_8cons_nssec5_SC[, .(observations = .N),
                                         by = c("synthetic_zone","nssec_sp")]
data.table::setkey(sp_8cons_nssec5_SC, synthetic_zone)
sp_8cons_nssec5_SC <- data.table::dcast(sp_8cons_nssec5_SC, synthetic_zone ~ nssec_sp,
                                        value.var = "observations")

sp_8cons_nssec5_SC <- merge(sp_8cons_nssec5_SC, data_nssec_sco, 
                            by.x = "synthetic_zone", by.y = "areacode")
sp_8cons_nssec5_SC[, diff_management_professional := sp_management_professional - 
                     census_management_professional]
sp_8cons_nssec5_SC[, diff_intermediate := sp_intermediate - census_intermediate]
sp_8cons_nssec5_SC[, diff_small_employers_own_account := sp_small_employers_own_account - 
                     census_small_employers_own_account]
sp_8cons_nssec5_SC[, diff_lower_supervisory_technical := sp_lower_supervisory_technical - 
                     census_lower_supervisory_technical]
sp_8cons_nssec5_SC[, diff_semi_routine_routine := sp_semi_routine_routine - 
                     census_semi_routine_routine]
sp_8cons_nssec5_SC[, diff_other := sp_other - census_other]

sp_8cons_nssec5_SC <- sp_8cons_nssec5_SC[, .(synthetic_zone,
        diff_management_professional, diff_intermediate,
        diff_small_employers_own_account, diff_lower_supervisory_technical,
        diff_semi_routine_routine, diff_other)]
sp_8cons_nssec5_SC <- melt(sp_8cons_nssec5_SC, id.var = c("synthetic_zone"))
sp_8cons_nssec5_SC[, sp := as.factor("8cons")]

# ---------------------------------------- #

# visualise differences 
sp_nssec5_comp_plot <- ggplot(sp_8cons_nssec5_SC,) +
  geom_histogram(aes(x = value), alpha = definitions$dot_alpha,
                 col = definitions$supp_col, fill = "darkgrey", binwidth = 10) +
  labs(x = "difference in number of individuals per DZ",
       y = "number of of DZ",
       caption = ("")) +
  xlim(-600,600) +
  facet_wrap(~ variable , ncol = 3, scales = "fixed",
             labeller = labeller(variable = 
    c("diff_management_professional"  = "management & professional",
      "diff_intermediate"  = "intermediate",
      "diff_small_employers_own_account"  = "small employers & own account",
      "diff_lower_supervisory_technical"  = "lower supervisory & technical",
      "diff_semi_routine_routine"  = "semi-routine & routine",
      "diff_other"  = "all other"))) + 
  geom_vline(xintercept = 0, size = 0.25) + 
  labs(title = "comparison for the population of working age (16-74 years)") +
  theme_classic() +
  annotate("rect", xmin = 0, xmax = Inf, ymin = -Inf, ymax = Inf,
           alpha = definitions$sep_alpha, fill = definitions$sep_cols[1]) + 
  annotate("rect", xmin = -Inf, xmax = 0, ymin = -Inf, ymax = Inf,
           alpha = definitions$sep_alpha, fill = definitions$sep_cols[2]) +
  geom_text(x = 300, y = 2500,  label = "SP overestimates", size = 3) + 
  geom_text(x = -300, y = 2500, label = "SP underestimates", size = 3)
ggplot2::ggsave(sp_nssec5_comp_plot, filename = "ROutput/sp_nssec5_comp_plot.jpeg",
                units = "cm", width =  definitions$fig_width, height = definitions$fig_height)  

# ---------------------------------------------------------------------------- #
# ---------------------------------------------------------------------------- #